package org.jbpm.examples.concurrency.forktest;

import org.jbpm.api.ProcessInstance;
import org.jbpm.pvm.internal.model.ExecutionImpl;
import org.jbpm.test.JbpmTestCase;

import java.util.Set;

public class ForkResultsInNullEndActivity extends JbpmTestCase {
    private String deploymentId;

    protected void setUp() throws Exception {
        super.setUp();
        final String jpdl = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<process name=\"ForkTest\" xmlns=\"http://jbpm.org/4.3/jpdl\">\n" +
                "    <start name=\"preparing test\">\n" +
                "        <transition to=\"fork\"/>\n" +
                "    </start>\n" +
                "    <fork name=\"fork\">\n" +
                "        <transition to=\"state1\"/>\n" +
                "        <transition to=\"state2\"/>\n" +
                "        <transition to=\"cancel\"/>\n" +
                "    </fork>\n" +
                "    <state name=\"cancel\">\n" +
                "        <transition to=\"cancelled\"/>\n" +
                "    </state>\n" +
                "    <state name=\"state1\">\n" +
                "        <transition to=\"states completed\"/>\n" +
                "    </state>\n" +
                "    <state name=\"state2\">\n" +
                "        <transition to=\"states completed\"/>\n" +
                "    </state>\n" +
                "    <join multiplicity=\"2\" name=\"states completed\">\n" +
                "        <transition to=\"completed\"/>\n" +
                "    </join>\n" +
                "    <end name=\"cancelled\"/>\n" +
                "    <end name=\"completed\"/>\n" +
                "</process>";

        // Deploys the process
        deploymentId = repositoryService.createDeployment().
                addResourceFromString("ForkTest.jpdl.xml", jpdl).deploy();
    }

    protected void tearDown() throws Exception {
        repositoryService.deleteDeploymentCascade(deploymentId);
        super.tearDown();
    }

    private void verifyInitialActivities(ProcessInstance proc) {
        String[] expectedStartActivities = {"state1", "state2", "cancel"};
        Set<String> foundStartActivities = proc.findActiveActivityNames();
        for (String act : expectedStartActivities) {
            assertTrue(foundStartActivities.contains(act));
        }
    }

    public void testCompleteJoin() {
        ProcessInstance proc = executionService.startProcessInstanceByKey("ForkTest");
        verifyInitialActivities(proc);

        String state1Id = proc.findActiveExecutionIn("state1").getId();
        proc = executionService.signalExecutionById(state1Id);

        String state2Id = proc.findActiveExecutionIn("state2").getId();
        proc = executionService.signalExecutionById(state2Id);

        assertExecutionEnded(proc.getId());

        //the name of the end activity should now be set on the execution
        ExecutionImpl executionImpl = (ExecutionImpl) proc;
        assertEquals("completed", executionImpl.getActivityName());
    }

    public void testCancelWithoutJoin() {
        ProcessInstance proc = executionService.startProcessInstanceByKey("ForkTest");
        verifyInitialActivities(proc);

        String driveTruckExecutionId = proc.findActiveExecutionIn("cancel").getId();
        proc = executionService.signalExecutionById(driveTruckExecutionId);

        assertExecutionEnded(proc.getId());

        //the name of the end activity should now be set on the execution
        ExecutionImpl executionImpl = (ExecutionImpl) proc;
        assertEquals("cancelled", executionImpl.getActivityName());
    }

}
